/***************************************************************
   LF2 Data Changer is a program that a allows you to edit, LF2 data files.
    Copyright (C) 2012  Someone else

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

My email address is <adrianbibbywalther@gmail.com>.
If you need any help, contact me there, or on the LFE Forums.
 **************************************************************/

#include "EditorPanel.h"
#include "LF2DataChangerMain.h"

#include <wx/font.h>
#include <wx/fontenum.h>
#include <wx/filename.h>

//(*InternalHeaders(EditorPanel)
#include <wx/intl.h>
#include <wx/string.h>
//*)

//(*IdInit(EditorPanel)
const long EditorPanel::ID_PANELEDITOR = wxNewId();
const long EditorPanel::ID_PANELBUILDER = wxNewId();
//*)
const long EditorPanel::ID_EDITOR= wxNewId();

BEGIN_EVENT_TABLE(EditorPanel,wxPanel)
    //(*EventTable(EditorPanel)
    //*)
END_EVENT_TABLE()

EditorPanel::EditorPanel(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size){
    Frame = static_cast<LF2DataChangerFrame*>(parent->GetParent());
    Saved = false;

    //(*Initialize(EditorPanel)
    wxBoxSizer* Sizer;
    wxBoxSizer* EditorSizer;

    Create(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("wxID_ANY"));
    Sizer = new wxBoxSizer(wxVERTICAL);
    PanelEditor = new wxPanel(this, ID_PANELEDITOR, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANELEDITOR"));
    EditorSizer = new wxBoxSizer(wxHORIZONTAL);
    PanelEditor->SetSizer(EditorSizer);
    EditorSizer->Fit(PanelEditor);
    EditorSizer->SetSizeHints(PanelEditor);
    Sizer->Add(PanelEditor, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    PanelBuilder = new wxPanel(this, ID_PANELBUILDER, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANELBUILDER"));
    PanelBuilder->Hide();
    Sizer->Add(PanelBuilder, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SetSizer(Sizer);
    Sizer->Fit(this);
    Sizer->SetSizeHints(this);
    //*)

    Editor = new wxStyledTextCtrl(PanelEditor,ID_EDITOR,wxDefaultPosition,wxDefaultSize,wxNO_BORDER|wxVSCROLL,_T("ID_EDITOR"));
    Editor->SetBackSpaceUnIndents(true);
    Editor->SetUseTabs(false);
    Editor->SetCaretLineVisible(true);
    Editor->SetCaretLineBackground(wxColour(232,232,232));

    wxFontEnumerator __FontEnumerator;
    __FontEnumerator.EnumerateFacenames();
    #if wxCHECK_VERSION(2, 8, 0)
        const wxArrayString& __FontFaces = __FontEnumerator.GetFacenames();
    #else
        const wxArrayString& __FontFaces = *__FontEnumerator.GetFacenames();
    #endif

    wxString __EditorFontFace;
    if ( __FontFaces.Index(_T("Courier New")) != wxNOT_FOUND )
        __EditorFontFace = _T("Courier New");
    else if ( __FontFaces.Index(_T("Lucida Console")) != wxNOT_FOUND )
        __EditorFontFace = _T("Lucida Console");
    else if ( __FontFaces.Index(_T("Courier")) != wxNOT_FOUND )
        __EditorFontFace = _T("Courier");

    EditorDEFAULTFont.Create(10,wxMODERN,wxFONTSTYLE_NORMAL,wxNORMAL,false,__EditorFontFace,wxFONTENCODING_DEFAULT);
    EditorBOLDFont.Create(10,wxMODERN,wxFONTSTYLE_NORMAL,wxBOLD,false,__EditorFontFace,wxFONTENCODING_DEFAULT);
    EditorBIGFont.Create(12,wxMODERN,wxFONTSTYLE_NORMAL,wxBOLD,false,__EditorFontFace,wxFONTENCODING_DEFAULT);

    Editor->StyleSetFont(wxSTC_STYLE_DEFAULT,EditorDEFAULTFont);
    Editor->SetMarginType(0,wxSTC_MARGIN_NUMBER);

    wxString Str;
    Str << Editor->GetLineCount();
    unsigned int Length = Str.length();
    if(Length<4) Length = 4;

    Editor->SetMarginWidth(0,(36+Editor->GetZoom()*3)*Length/4);

    Editor->StyleSetForeground(wxSTC_STYLE_LINENUMBER,wxColour(96,96,96));
    Editor->SetFocus();
    StyleEditor();
    EditorSizer->Add(Editor, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);

    Connect(ID_EDITOR,wxEVT_STC_CHANGE,(wxObjectEventFunction)&EditorPanel::OnChange);
    Connect(ID_EDITOR,wxEVT_STC_CHARADDED,(wxObjectEventFunction)&EditorPanel::OnCharAdded);
    Connect(ID_EDITOR,wxEVT_STC_MARGINCLICK,(wxObjectEventFunction)&EditorPanel::OnMarginClick);
    Connect(ID_EDITOR,wxEVT_STC_UPDATEUI,(wxObjectEventFunction)&EditorPanel::OnEditorUpdateUI);
    Connect(ID_EDITOR,wxEVT_STC_ZOOM,(wxObjectEventFunction)&EditorPanel::OnZoom);

    Editor->SetEOLMode(wxSTC_EOL_LF);

    #ifdef USE_WRITETOEXE
        Pid = -1;
        Id = 0;
        Type = 0;
        Addr = (char*)0xffffffff;
    #endif
}

EditorPanel::~EditorPanel(){
    //(*Destroy(EditorPanel)
    //*)
}

wxString &EditorPanel::GetFilename(){
    return Filename;
}

void EditorPanel::StyleEditor(){
    Editor->SetMarginType(1,wxSTC_MARGIN_SYMBOL);
    Editor->SetMarginWidth(1,16);
    Editor->SetMarginMask(1,wxSTC_MASK_FOLDERS);
    Editor->SetMarginSensitive(1,true);

    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDER       ,wxSTC_MARK_BOXPLUS          ,_T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDEREND    ,wxSTC_MARK_BOXPLUSCONNECTED ,_T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDERMIDTAIL,wxSTC_MARK_TCORNER,          _T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDEROPEN   ,wxSTC_MARK_BOXMINUS         ,_T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDEROPENMID,wxSTC_MARK_BOXMINUSCONNECTED,_T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDERSUB    ,wxSTC_MARK_VLINE,            _T("WHITE"),_T("GRAY"));
    Editor->MarkerDefine(wxSTC_MARKNUM_FOLDERTAIL   ,wxSTC_MARK_LCORNER,          _T("WHITE"),_T("GRAY"));
    Editor->SetFoldFlags(wxSTC_FOLDFLAG_LINEAFTER_CONTRACTED);
}

void EditorPanel::UpdateLexer(int Lexer){
    Editor->StyleClearAll();

    Editor->SetLexer(Lexer);

    switch(Editor->GetLexer()){
        case wxSTC_LEX_LF2AI: {
            Editor->SetIndent(4);
            Editor->SetTabWidth(4);

            Editor->SetProperty(_T("fold"),_T("1"));
            Editor->SetProperty(_T("fold.comment"),_T("1"));
            Editor->SetProperty(_T("fold.preprocessor"),_T("1"));
            Editor->SetProperty(_T("fold.compact"),_T("0"));
            Editor->SetProperty(_T("fold.at.else"),_T("0"));

            Editor->SetKeyWords(0,_T("if else return for do while switch case default break true false void const cast in out inout "
                                     "bool int int8 int16 int32 int64 uint uint8 uint16 uint32 uint64 float double string array namespace interface class mixin "
                                     "final super override private enum typedef funcdef import from is null get set shared and not or xor "
                                     "BoolArray CharArray CharArrayArray IntArray ItrArray BdyArray DataFileArray ObjectArray FrameArray BackgroundArray SpawnArray PhaseArray StageArray "
                                     "Info Opoint Bpoint Cpoint Wpoint Itr Bdy Frame DataFile Object Background Spawn Phase Stage FileManager Game"));

            Editor->SetKeyWords(1,_T("clr print loadTarget up down left right A J D DrA DlA DuA DdA DrJ DlJ DuJ DdJ DJA rand mode "
                                     "fpFromIEEE cos sin tan acos asin atan atan2 cosh sinh tanh log log10 pow sqrt ceil abs floor fraction "
                                     "difficulty background bg_width bg_zwidth1 bg_zwidth2 x y z facing holding_up holding_down holding_left holding_right holding_a holding_j holding_d "
                                     "up down left right A J D hp dark_hp max_hp mp frame bdefend fall team id ego blink state weapon_type weapon_held "
                                     "shake num ctimer arest vrest x_velocity y_velocity z_velocity clone type reserve stage_bound stage_clear "

                                     "opNeg opCom opPreInc opPreDec opPostInc opPostDec opEquals opCmp opAssign opAddAssign opSubAssign opMulAssign opDivAssign opModAssign "
                                     "opAndAssign opOrAssign opXorAssign opShlAssign opShrAssign opUShrAssign get_opIndex set_opIndex "

                                     "insertAt removeAt insertLast removeLast length reserve resize sortAsc sortDesc reverse find isEmpty "
                                     "split substr findFirst findLast formatInt formatFloat parseInt parseFloat "

                                     "unkwn1 unkwn2 unkwn3 unkwn4 unkwn5 unkwn6 unkwn7 unkwn8 unkwn9 unkwn10 unkwn11 unkwn12 unkwn13 unkwn14 unkwn15 unkwn16 unkwn17 unkwn18 unkwn19 "
                                     "kind action dvx dvy dvz oid injury cover vaction aaction jaction daction throwvx throwvy hurtable decrease dircontrol taction throwinjury throwvz "
                                     "weaponact attacking w h effect catchingact1 catchingact2 caughtact1 caughtact2 zwidth exists pic wait next hit_a hit_d hit_j "
                                     "hit_Fa hit_Ua hit_Da hit_Fj hit_Uj hit_Dj hit_ja centerx centery opoint bpoint cpoint wpoint itr_count bdy_count itrs bdys "
                                     "itr_x itr_y itr_w bdy_h bdy_x bdy_y bdy_w bdy_h fname walking_frame_rate walking_speed walking_speedz running_frame_rate running_speed running_speedz "
                                     "heavy_walking_speed heavy_walking_speedz heavy_running_speed heavy_running_speedz jump_height jump_distance jump_distancez dash_height dash_distance "
                                     "rowing_height rowing_distance weapon_hp weapon_drop_hurt pic_count frames name x_acceleration y_acceleration z_acceleration "
                                     "pic_bmps head_bmp small_bmp pic_index pic_width pic_height pic_row pic_col move_counter run_counter weapon_holder weapon_thrower "
                                     "x_real y_real z_real frame1 frame2 frame3 frame4 wait_counter ccatcher attacked_object_num pic_gain bottle_hp firzen_counter armour_multiplier total_attack enemy data "
                                     "perspective1 perspective2 shadow1 shadow2 layer_count shadow_bmp layer_bmps transparency layer_width layer_x layer_y layer_height datas backgrounds objects files "
                                     "stages phase_count phases bound music spawns times reserve join join_reserve act ratio role when_clear_goto_phase data_ptrs"));

            Editor->SetKeyWords(2,_T("game self target"));

            Editor->StyleSetForeground(wxSTC_LF2AI_COMMENT,wxColour(128,128,128));
            Editor->StyleSetForeground(wxSTC_LF2AI_COMMENTLINE,wxColour(128,128,128));

            Editor->StyleSetForeground(wxSTC_LF2AI_NUMBER,wxColour(240,0,240));
            Editor->StyleSetForeground(wxSTC_LF2AI_STRING,wxColour(0,0,255));
            Editor->StyleSetForeground(wxSTC_LF2AI_STRINGVAR,wxColour(0,0,255));
            Editor->StyleSetForeground(wxSTC_LF2AI_MULTISTRING,wxColour(0,0,255));
            Editor->StyleSetForeground(wxSTC_LF2AI_PREPROCESSOR1,wxColour(0,160,0));
            Editor->StyleSetForeground(wxSTC_LF2AI_PREPROCESSOR2,wxColour(0,160,0));
            Editor->StyleSetForeground(wxSTC_LF2AI_PREPROCESSORSTRING,wxColour(0,160,255));
            Editor->StyleSetForeground(wxSTC_LF2AI_OPERATOR,wxColour(255,0,0));

            Editor->StyleSetForeground(wxSTC_LF2AI_KEYWORD,wxColour(0,0,128));
            Editor->StyleSetForeground(wxSTC_LF2AI_KEYWORD2,wxColour(128,0,0));
            Editor->StyleSetForeground(wxSTC_LF2AI_KEYWORD3,wxColour(0,128,0));

            Editor->StyleSetForeground(wxSTC_STYLE_BRACEBAD,wxColour(128,0,128));

            Editor->StyleSetForeground(wxSTC_STYLE_BRACELIGHT,wxColour(255,0,0));

            Editor->StyleSetFont(wxSTC_LF2AI_KEYWORD,EditorBOLDFont);
            Editor->StyleSetFont(wxSTC_LF2AI_KEYWORD3,EditorBOLDFont);
            Editor->StyleSetFont(wxSTC_LF2AI_STRINGVAR,EditorBOLDFont);
            Editor->StyleSetFont(wxSTC_STYLE_BRACELIGHT,EditorBIGFont);
            break;
        }
        case wxSTC_LEX_LF2DC: {
            Editor->SetIndent(3);
            Editor->SetTabWidth(3);

            Editor->SetProperty(_T("fold"),_T("1"));

            Editor->SetKeyWords(0,_T("<bmp_begin> <frame> <stage> <phase> <object> <file_editing> <background> <weapon_strength_list> "
                                     "<bmp_end> <frame_end> <stage_end> <phase_end> <object_end> <file_editing_end> <background_end> <weapon_strength_list_end> "
                                     "<soldier> <boss> <end>"));

            Editor->SetKeyWords(1,_T("bpoint: wpoint: bdy: itr: cpoint: opoint: layer: "
                                     "bpoint_end: wpoint_end: bdy_end: itr_end: cpoint_end: opoint_end: layer_end"));

            Editor->SetKeyWords(2,_T("name: head: small: walking_frame_rate walking_speed walking_speedz "
                                     "running_frame_rate running_speed running_speedz heavy_walking_speed heavy_walking_speedz "
                                     "heavy_running_speed heavy_running_speedz jump_height jump_distance jump_distancez "
                                     "dash_height dash_distance dash_distancez rowing_height rowing_distance pic: state: wait: mp: "
                                     "next: dvx: dvy: dvz: centerx: centery: hit_a: hit_d: hit_j: hit_Fa: hit_Fj: hit_Ua: hit_Uj: hit_Da: hit_Dj: hit_ja: "
                                     "kind: x: y: row: col: w: h: vaction: aaction: taction: throwvx: throwvy: throwvz: "
                                     "hurtable: throwinjury: decrease: weaponact: attacking: cover: catchingact: caughtact: "
                                     "fronthurtact: backhurtact: dircontrol: sound: fall: bdefend: injury: arest: vrest: zwidth: effect: "
                                     "action: oid: facing: id: type: bound: music: hp: times: ratio: when_clear_goto_phase: reserve: join: "
                                     "act: rect: width: height: zboundary: shadow: shadowsize: transparency: loop: cc: c1: c2: "
                                     "weapon_hp: weapon_drop_hurt: weapon_hit_sound: weapon_drop_sound: weapon_broken_sound: entry:"));

            Editor->SetKeyWords(3,_T("<bmp_begin> <frame> <stage> <phase> <object> <file_editing> <background> <weapon_strength_list> "
                                     "bpoint: wpoint: bdy: itr: cpoint: opoint: layer:"));

            Editor->SetKeyWords(4,_T("<bmp_end> <frame_end> <stage_end> <phase_end> <object_end> <file_editing_end> <background_end> <weapon_strength_list_end> "
                                     "bpoint_end: wpoint_end: bdy_end: itr_end: cpoint_end: opoint_end: layer_end"));

            Editor->StyleSetForeground(wxSTC_LF2DC_COMMENT,wxColour(128,128,128));
            Editor->StyleSetForeground(wxSTC_LF2DC_NUMBER,wxColour(240,0,240));

            Editor->StyleSetForeground(wxSTC_LF2DC_KEYWORD,wxColour(0,0,255));
            Editor->StyleSetForeground(wxSTC_LF2DC_KEYWORD2,wxColour(0,128,0));
            Editor->StyleSetForeground(wxSTC_LF2DC_KEYWORD3,wxColour(128,0,0));
            break;
        }
    }

    Editor->Colourise(-1,Editor->GetText().length()+1);
}

void EditorPanel::OnMarginClick(wxStyledTextEvent& event){
    if(event.GetMargin() == 1){
        int lineClick = Editor->LineFromPosition(event.GetPosition());
        int levelClick = Editor->GetFoldLevel(lineClick);
        if((levelClick & wxSTC_FOLDLEVELHEADERFLAG)>0){
            Editor->ToggleFold(lineClick);
        }
    }
}

void EditorPanel::OnCharAdded(wxStyledTextEvent& event){
    char Char = (char)event.GetKey();
    if(Char == '\n' || Char == '\r'){
        int Line = Editor->GetCurrentLine();
        int Indent = 0;
        if(Line>0){
            Indent = Editor->GetLineIndentation(Line-1);
        }
        if(Indent == 0) return;
        Editor->SetLineIndentation(Line,Indent);
        Editor->GotoPos(Editor->PositionFromLine(Line)+Indent);
    }
}

void EditorPanel::OnChange(wxStyledTextEvent& event){
    if(Editor->CanUndo()){
        Frame->AuiToolBarEdit->EnableTool(Frame->ID_AUITOOLBAREDITUNDO,true);
        Frame->MenuEditUndo->Enable(true);
    }else{
        Frame->AuiToolBarEdit->EnableTool(Frame->ID_AUITOOLBAREDITUNDO,false);
        Frame->MenuEditUndo->Enable(false);
    }
    if(Editor->CanRedo()){
        Frame->AuiToolBarEdit->EnableTool(Frame->ID_AUITOOLBAREDITREDO,true);
        Frame->MenuEditRedo->Enable(true);
    }else{
        Frame->AuiToolBarEdit->EnableTool(Frame->ID_AUITOOLBAREDITREDO,false);
        Frame->MenuEditRedo->Enable(false);
    }
    Frame->AuiToolBarEdit->Realize();

    if(Saved == !Editor->GetModify()) return;
    Saved = !Editor->GetModify();

    wxString Name;

    if(Filename.find(_T("/")) >= Filename.length()){
        Name = _("Untitled");
        Name.append(Filename);
    }else{
        Name = Filename.substr(Filename.rfind(_T("/"))+1);
    }
    if(!Saved) Name.Prepend(_T("*"));

    Frame->SetTabText(this,Name);
    Frame->UpdateTitle();

    if(Filename.find(_T("/")) >= Filename.length()) return;
    Frame->AuiToolBarFile->EnableTool(Frame->ID_AUITOOLBARFILESAVE,!Saved);
    Frame->MenuFileSave->Enable(!Saved);
    Frame->AuiToolBarFile->Realize();
}

void EditorPanel::OnZoom(wxStyledTextEvent& event){
    wxString Str;
    Str << Editor->GetLineCount();
    unsigned int Length = Str.length();
    if(Length<4) Length = 4;

    Editor->SetMarginWidth(0,(36+Editor->GetZoom()*3)*Length/4);
}

void EditorPanel::OnEditorUpdateUI(wxStyledTextEvent& event){
    wxString Str;
    Str << Editor->GetLineCount();
    unsigned int Length = Str.length();
    if(Length<4) Length = 4;

    HighlightBraces();

    Editor->SetMarginWidth(0,(36+Editor->GetZoom()*3)*Length/4);
    Frame->UpdateStatusBar();
    Frame->UpdateSearchBar();
}

void EditorPanel::HighlightBraces(){
    switch(Editor->GetLexer()){
        case wxSTC_LEX_LF2AI: {//pretty much taken from Code::Blocks source
            int currPos = Editor->GetCurrentPos();
            int newPos = Editor->BraceMatch(currPos);
            if(newPos == -1){
                if(currPos>0) currPos--;
                newPos = Editor->BraceMatch(currPos);
            }
            wxChar ch = Editor->GetCharAt(currPos);
            if(ch == _T('{') || ch == _T('[') || ch == _T('(') || ch == _T('}') || ch == _T(']') || ch == _T(')')){
                if(newPos != -1){
                    Editor->BraceHighlight(currPos,newPos);
                    const int currColum = Editor->GetColumn(currPos);
                    const int newColum = Editor->GetColumn(newPos);
                    Editor->SetHighlightGuide((currColum<newColum)?currColum:newColum);
                }else{
                    Editor->BraceBadLight(currPos);
                }
            }else{
                Editor->BraceHighlight(-1,-1);
            }
            break;
        }
        case wxSTC_LEX_LF2DC: {
            //will implement this later
            break;
        }
    }
}
