/***************************************************************
   LF2 Data Changer is a program that a allows you to edit, LF2 data files.
    Copyright (C) 2012  Someone else

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

My email address is <adrianbibbywalther@gmail.com>.
If you need any help, contact me there, or on the LFE Forums.
 **************************************************************/

#ifndef LF2DATACHANGERMAIN_H
#define LF2DATACHANGERMAIN_H

#include <map>
#include <vector>
#include "tinyxml.h"
#include "LF2DataChangerApp.h"
#include "EditorPanel.h"
#include "AboutBox.h"
#ifdef USE_WRITETOEXE
    #include "WriteToDialog.h"
    #include "TransDialog.h"
#endif

#include "DDE.h"

//(*Headers(LF2DataChangerFrame)
#include <wx/srchctrl.h>
#include <wx/sizer.h>
#include <wx/menu.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/aui/aui.h>
#include <wx/radiobut.h>
#include <wx/panel.h>
#include <wx/filedlg.h>
#include <wx/button.h>
#include <wx/frame.h>
#include <wx/statusbr.h>
//*)

class wxCustomAuiNotebook:public wxAuiNotebook {
    public:
        wxCustomAuiNotebook():wxAuiNotebook(){}
        wxCustomAuiNotebook(wxWindow* parent,wxWindowID id = wxID_ANY,const wxPoint& pos = wxDefaultPosition,const wxSize& size = wxDefaultSize,long style = wxAUI_NB_DEFAULT_STYLE):
            wxAuiNotebook(parent,id,pos,size,style){}

    friend class LF2DataChangerFrame;
};

//#define wxAuiNotebook wxCustomAuiNotebook//so it still works with wxSmith

#ifdef USE_WRITETOEXE
    struct ProcStruct {
        HWND Window;
        int Pid;
    };
#endif

class EditorPanel;

class LF2DataChangerFrame: public wxFrame {
    friend class EditorPanel;

    public:
        LF2DataChangerFrame(wxWindow* parent,wxWindowID id = -1);
        virtual ~LF2DataChangerFrame();

        void SetTabText(unsigned int Index,const wxString &Text);
        void SetTabText(EditorPanel *Panel,const wxString &Text);

        bool LoadFile(wxString Filename,bool Copy = false,bool Alert = true);

        void UpdateSearchBar();
        void UpdateStatusBar();
        void UpdateTitle();

        void UpdateMenuFileTemplate();

        #ifdef USE_WRITETOEXE
            void UpdateMenuWriteChoose();
        #endif

        bool Destroy();

    private:
        void OnDropFiles(wxDropFilesEvent &event);

        //(*Handlers(LF2DataChangerFrame)
        void OnQuit(wxCommandEvent& event);
        void OnAbout(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnShowFile(wxCommandEvent& event);
        void OnShowEdit(wxCommandEvent& event);
        void OnShowSend(wxCommandEvent& event);
        void OnShowSearch(wxCommandEvent& event);
        void OnRedo(wxCommandEvent& event);
        void OnUndo(wxCommandEvent& event);
        void OnCut(wxCommandEvent& event);
        void OnCopy(wxCommandEvent& event);
        void OnPaste(wxCommandEvent& event);
        void OnToolSearch(wxCommandEvent& event);
        void OnToolReplace(wxCommandEvent& event);
        void OnPageChanged(wxAuiNotebookEvent& event);
        void OnBgDClick(wxAuiNotebookEvent& event);
        void OnTabMiddleUp(wxAuiNotebookEvent& event);
        void OnNew(wxCommandEvent& event);
        void OnOpen(wxCommandEvent& event);
        void OnSave(wxCommandEvent& event);
        void OnSaveAs(wxCommandEvent& event);
        void OnCloseCurrentTab(wxCommandEvent& event);
        void OnNewFile(wxCommandEvent& event);
        void OnPageClose(wxAuiNotebookEvent& event);
        void OnPageClosed(wxAuiNotebookEvent& event);
        void OnSearch(wxCommandEvent& event);
        void OnReplace(wxCommandEvent& event);
        void OnReplaceAll(wxCommandEvent& event);
        void OnPaneClose(wxAuiManagerEvent& event);
        void OnWriteToExe(wxCommandEvent& event);
        void OnWriteToSelection(wxCommandEvent& event);
        void OnAutoWrite(wxCommandEvent& event);
        void OnTabRightUp(wxAuiNotebookEvent& event);
        void OnOpenFile(wxCommandEvent& event);
        //*)
        void OnPrepareNew(wxAuiToolBarEvent& event);
        void OnNewTemplate(wxCommandEvent& event);

        void OnPrepareOpen(wxAuiToolBarEvent& event);
        void OnOpenRecent(wxCommandEvent& event);

        void OnPrepareWriteToExe(wxAuiToolBarEvent& event);

        void OnMenuOpen(wxMenuEvent& event);
        void OnMenuHighlight(wxMenuEvent& event);

        //(*Identifiers(LF2DataChangerFrame)
        static const long ID_AUITOOLBARFILENEW;
        static const long ID_AUITOOLBARFILEOPEN;
        static const long ID_AUITOOLBARFILESAVE;
        static const long ID_AUITOOLBARFILESAVEAS;
        static const long ID_AUITOOLBARFILECLOSE;
        static const long ID_AUITOOLBARFILE;
        static const long ID_AUITOOLBAREDITUNDO;
        static const long ID_AUITOOLBAREDITREDO;
        static const long ID_AUITOOLBAREDITCUT;
        static const long ID_AUITOOLBAREDITCOPY;
        static const long ID_AUITOOLBAREDITPASTE;
        static const long ID_AUITOOLBAREDITSEARCH;
        static const long ID_AUITOOLBAREDITREPLACE;
        static const long ID_AUITOOLBAREDIT;
        static const long ID_SEARCHDIALOGSEARCHFIELD;
        static const long ID_SEARCHDIALOGSEARCHBUTTON;
        static const long ID_SEARCHDIALOGSEARCHREPLACE;
        static const long ID_SEARCHDIALOGREPLACEBUTTON;
        static const long ID_SEARCHDIALOGREPLACEALLBUTTON;
        static const long ID_SEARCHDIALOGWHOLEWORD;
        static const long ID_SEARCHDIALOGUP;
        static const long ID_SEARCHDIALOGMATCHCASE;
        static const long ID_SEARCHDIALOGDOWN;
        static const long ID_SEARCHDIALOG;
        static const long ID_AUINOTEBOOK;
        static const long ID_AUITOOLBARSEARCHFIELD;
        static const long ID_AUITOOLBARSEARCHREPLACE;
        static const long ID_AUITOOLBARSEARCH;
        static const long ID_AUITOOLBARWRITEWRITE;
        static const long ID_AUITOOLBARWRITEAUTO;
        static const long ID_AUITOOLBARWRITE;
        static const long ID_MENUFILENEW;
        static const long ID_MENUFILETEMPLATEEMPTY;
        static const long ID_MENUFILETEMPLATE;
        static const long ID_MENUFILEOPEN;
        static const long ID_MENUFILERECENTEMPTY;
        static const long ID_MENUFILERECENT;
        static const long ID_MENUFILESAVE;
        static const long ID_MENUFILESAVEAS;
        static const long ID_MENUFILECLOSE;
        static const long ID_MENUFILEQUIT;
        static const long ID_MENUEDITUNDO;
        static const long ID_MENUEDITREDO;
        static const long ID_MENUEDITCUT;
        static const long ID_MENUEDITCOPY;
        static const long ID_MENUEDITPASTE;
        static const long ID_MENUEDITSEARCH;
        static const long ID_MENUEDITREPLACE;
        static const long ID_MENUVIEWFILE;
        static const long ID_MENUVIEWEDIT;
        static const long ID_MENUVIEWWRITE;
        static const long ID_MENUVIEWSEARCH;
        static const long ID_MENUWRITEWRITE;
        static const long ID_MENUWRITECHOOSEEMPTY;
        static const long ID_MENUWRITECHOOSE;
        static const long ID_MENUWRITEAUTO;
        static const long ID_MENUHELPABOUT;
        static const long ID_STATUSBAR;
        static const long ID_TABSAVE;
        static const long ID_TABSAVEAS;
        static const long ID_TABCLOSE;
        //*)

        //(*Declarations(LF2DataChangerFrame)
        wxAuiToolBar* AuiToolBarFile;
        wxMenu* MenuHelp;
        wxCheckBox* SearchDialogWholeWord;
        wxMenuItem* TabClose;
        wxSearchCtrl* AuiToolBarSearchReplace;
        wxMenu* MenuWrite;
        wxMenuItem* MenuViewFile;
        wxTextCtrl* SearchDialogSearchReplace;
        wxMenuItem* MenuEditUndo;
        wxMenuItem* MenuFileSaveAs;
        wxMenuItem* MenuEditPaste;
        wxAuiToolBar* AuiToolBarEdit;
        wxMenuItem* TabSaveAs;
        wxMenuItem* MenuHelpAbout;
        wxButton* SearchDialogReplaceButton;
        wxButton* SearchDialogSearchButton;
        wxAuiNotebook* AuiNotebook;
        wxButton* SearchDialogReplaceAllButton;
        wxAuiToolBar* AuiToolBarSearch;
        wxMenu* MenuFileTemplate;
        wxMenuItem* MenuEditRedo;
        wxCheckBox* SearchDialogMatchCase;
        wxMenuItem* MenuWriteChooseEmpty;
        wxMenuItem* MenuFileOpen;
        wxMenuItem* MenuEditCut;
        wxSearchCtrl* AuiToolBarSearchField;
        wxMenuItem* MenuFileNew;
        wxMenuItem* TabSave;
        wxPanel* SearchDialog;
        wxMenu* MenuFileRecent;
        wxFileDialog* SaveFileDlg;
        wxMenuItem* MenuFileTemplateEmpty;
        wxMenuItem* MenuFileClose;
        wxMenuItem* MenuFileQuit;
        wxMenu* MenuFile;
        wxMenuItem* MenuViewEdit;
        wxMenuItem* MenuEditReplace;
        wxMenu* MenuEdit;
        wxMenuItem* MenuViewWrite;
        wxMenuItem* MenuViewSearch;
        wxMenu TabMenu;
        wxMenuItem* MenuEditSearch;
        wxRadioButton* SearchDialogDown;
        wxTextCtrl* SearchDialogSearchField;
        wxMenuItem* MenuWriteAuto;
        wxMenuItem* MenuEditCopy;
        wxMenuItem* MenuFileSave;
        wxRadioButton* SearchDialogUp;
        wxAuiManager* AuiManager;
        wxFileDialog* OpenFileDlg;
        wxStatusBar* StatusBar;
        wxMenuItem* MenuWriteWrite;
        wxMenu* MenuView;
        wxAuiToolBar* AuiToolBarWrite;
        wxMenuItem* MenuFileRecentEmpty;
        wxMenu* MenuWriteChoose;
        //*)

        void LoadConfig();
        bool ShowHidePane(wxAuiPaneInfo &Pane);
        void CreateFile(wxString text = _T(""),wxString Filename = _T(""));
        void SaveFile(EditorPanel *Panel);
        void SaveFileAs(EditorPanel *Panel);

        wxString GetFilename(EditorPanel *Panel);

        void UpdateMenuFileRecent();
        void UpdateMenuFileRecent(const wxString &Filename);

        void DecryptString(std::string &String,unsigned int Length,wxString Key = _T("SiuHungIsAGoodBearBecauseHeIsVeryGood"));
        void EncryptString(std::string &String,unsigned int Length,wxString Key = _T("SiuHungIsAGoodBearBecauseHeIsVeryGood"),wxString Notice = _("This file was created using a datachanger made by Someone else\nIf you wish to try datachanging, visit http://lf-empire.de\n"));
        std::string GetXmlAttribute(TiXmlElement* Element,const std::string &Attribute,const std::string &Default);

        bool Searching;
        bool NewDropdownClicked;
        bool OpenDropdownClicked;

        bool DontUpdateMenuFileRecent;

        #ifdef USE_WRITETOEXE
            bool WriteDropdownClicked;
            bool PauseResumeThreadList(unsigned int dwOwnerPID,bool bResumeThread);

            static BOOL CALLBACK EnumWindowsProcAll(HWND hWnd,LPARAM lParam);
            static BOOL CALLBACK EnumWindowsProcAdv(HWND hWnd,LPARAM lParam);
            static wchar_t CurrentFilename[MAX_PATH];

            void WriteToExe(int Pid,int Id,int Type,char *Addr);
        #endif

        wxCmdLineParser *CmdParser;

        wxString ExePath;

        EditorPanel *CurrentPanel;
        AboutBox *AboutDlg;

        DDEServer *Server;

        #ifdef USE_WRITETOEXE
            WriteToDialog *WriteDlg;
            TransDialog *TransDlg;
        #endif

        std::map<long,wxString> TemplateFiles;

        std::vector<wxString> RecentFiles;
        std::map<long,unsigned int> RecentFilesMap;

        std::map<long,wxMenuItem*> SubMenus;

        #ifdef USE_WRITETOEXE
            std::map<long,ProcStruct> MatchingProcs;
        #endif

        DECLARE_EVENT_TABLE()
};

#endif // LF2DATACHANGERMAIN_H
